﻿namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    using Interpreter;
    using Stack;

    public abstract class ShoveExpression<T> : StatelessExpression
    {
        protected ShoveExpression(string stringRepresentation) : base(stringRepresentation)
        {
        }

        protected void Eval(IStack<T> stack, IStack<long> integerStack, int count = 1)
        {
            if (integerStack.Count == 0 ||
                integerStack.Top > stack.Count - count ||
                integerStack.Top < 0 ||
                stack.Count < 2)
                return;

            var index = (int) integerStack.Pop();
            var item = stack.Pop();

            stack.Insert(index, item);
        }
    }

    public class IntegerShoveExpression : ShoveExpression<long>
    {
        public IntegerShoveExpression() : base("INTEGER.SHOVE") { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.IntegerStack, interpreter.IntegerStack, 2);
        }
    }

    public class FloatShoveExpression : ShoveExpression<double>
    {
        public FloatShoveExpression() : base("FLOAT.SHOVE") { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.FloatStack, interpreter.IntegerStack);
        }
    }

    public class BooleanShoveExpression : ShoveExpression<bool>
    {
        public BooleanShoveExpression() : base("BOOLEAN.SHOVE") { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.BooleanStack, interpreter.IntegerStack);
        }
    }

    public class NameShoveExpression : ShoveExpression<string>
    {
        public NameShoveExpression() : base("NAME.SHOVE") { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.NameStack, interpreter.IntegerStack);
        }
    }

    public class ExecShoveExpression : ShoveExpression<Expression>
    {
        public ExecShoveExpression() : base("EXEC.SHOVE") { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.ExecStack, interpreter.IntegerStack);
        }
    }

    public class CodeShoveExpression : ShoveExpression<Expression>
    {
        public CodeShoveExpression() : base("CODE.SHOVE")
        {
        }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Eval(interpreter.CodeStack, interpreter.IntegerStack);
        }
    }
}