﻿namespace HeuristicLab.Algorithms.PushGP.Interpreter {
  using System.Collections.Generic;
  using System.Threading;
  using System.Threading.Tasks;
  using HeuristicLab.Algorithms.PushGP.Expressions;
  using HeuristicLab.Algorithms.PushGP.Stack;

  public interface IPushGpInterpreter {
    IStack<Expression> CodeStack { get; }

    IStack<Expression> ExecStack { get; }

    IStack<string> NameStack { get; }

    IStack<bool> BooleanStack { get; }

    IStack<long> IntegerStack { get; }

    IStack<double> FloatStack { get; }

    IDictionary<string, Expression> CustomExpressions { get; }

    Configuration Configuration { get; }

    bool IsNameQuoteFlagSet { get; set; }

    void Run(string code);

    Task RunAsync(string code, bool paused = false, CancellationToken token = default(CancellationToken));

    void Run(Expression program);

    Task RunAsync(Expression program, bool paused = false, CancellationToken token = default(CancellationToken));

    Task AbortAndResetAsync();

    Task AbortAsync();

    Task PauseAsync();

    Task ResumeAsync();

    void Step();

    void PrintStacks();
  }
}