﻿namespace HeuristicLab.BenchmarkSuite.ProblemData {
  using System;
  using System.Linq;

  using HeuristicLab.Algorithms.PushGP.Stack;

  public class Data<TIn, TOut> : IData {
    public readonly Example<TIn, TOut>[] Training;
    public readonly Example<TIn, TOut>[] Test;

    public Data(Example<TIn, TOut>[] training, Example<TIn, TOut>[] test, double bestResult, double worstResult) {
      this.Training = training;
      this.Test = test;
      this.WorstResult = worstResult;
      this.BestResult = bestResult;
    }

    public int TrainingCount { get { return Training.Length; } }
    public int TestCount { get { return Test.Length; } }

    public double WorstResult { get; private set; }
    public double BestResult { get; private set; }

    protected double CompareOutput<TStack>(IStack<TStack> stack, int index, Example<TIn, TOut>[] data, Func<TOut, double> differ) {
      if (stack.IsEmpty)
        return this.WorstResult;

      var diff = differ(data[index].Output);
      return diff == long.MinValue ? this.WorstResult : Math.Min(Math.Abs(diff), this.WorstResult);
    }

    protected static long[] GetAsciiValues(string str) {
      return str.Select(c => (long)c).ToArray();
    }

    protected static double GetAsciiResultDifference(string data, IStack<long> stack) {
      var resultCount = Math.Min(data.Length, stack.Count);
      var resultValues = stack.Pop(resultCount);

      return Math.Abs(GetAsciiValues(data).Sum() - resultValues.Sum());
    }

    protected static double GetFloatResultDifference(double data, IStack<double> stack) {
      return Math.Abs(data - stack.Pop());
    }

    protected static double GetIntResultDifference(long data, IStack<long> stack) {
      return Math.Abs(data - stack.Pop());
    }

    protected static double GetIntResultDifference(long[] data, IStack<long> stack) {
      var resultCount = Math.Min(data.Length, stack.Count);
      var resultValues = stack.Pop(resultCount);

      return Math.Abs(data.Sum() - resultValues.Sum());
    }
  }
}
